// Programm zur Demonstration eines Kopierkonstruktors
#include <iostream>
#include <vector>
using namespace std;

class DatenKlasse
{
public:
    DatenKlasse() // Konstruktor: erzeugt externe Daten
    {
        zeiger = new int;
        *zeiger = 5;
    }
    ~DatenKlasse() // Destruktor: gibt externe Daten frei
    {
        delete zeiger;
        zeiger = 0;
    }

    DatenKlasse(DatenKlasse&& data) // Move-Konstruktor
    {
        cout << "Move-Konstruktor" << endl;
        zeiger = data.zeiger;
        data.zeiger = nullptr;
    }

    DatenKlasse& operator=(DatenKlasse&& data) // Move-Zuweisung
    {
        cout << "Move-Zuweisung" << endl;
        zeiger = data.zeiger;
        data.zeiger = nullptr;
        return *this;
    }

    DatenKlasse(const DatenKlasse& data) // Kopierkonstruktor
    {
        // zur Demonstration meldet er sich
        cout << "Kopierkonstruktor" << endl;
        // Externe Daten erzeugen und kopieren
        zeiger = new int;
        *zeiger = *data.zeiger;
        // Normale Datenelemente auch kopieren
        sonstiges = data.sonstiges;
    }

    DatenKlasse& operator=(const DatenKlasse& data) // Zuweisung
    {
        cout << "Zuweisung" << endl;
        if (this != &data) { // keine Selbstzuweisung?
            // Externe Daten erzeugen und kopieren
            zeiger = new int;
            *zeiger = *data.zeiger;
            // Normale Datenelemente auch kopieren
            sonstiges = data.sonstiges;
        }
        return *this;
    }

    int  getData() { if (zeiger!=nullptr) return *zeiger; }
    void setData(int wert) {*zeiger=wert; }
private:
    int sonstiges; // steht fuer die Nicht-Zeiger Datenelemente
    int *zeiger;   // Zeiger, also Kopierkonstruktor notwendig
};

// Die Funktion dient nur zur Demonstration. Da der Parameter
// per Wert uebergeben wird, wird beim Aufruf der
// Kopierkonstruktor aufgerufen
DatenKlasse tuIrgendwas(DatenKlasse para)
{
    cout << "tuIrgendwas:" << para.getData() << endl;
    return para; // keine Referenz: Move-Konstruktor
}

// Hauptprogramm zum Testen
int main()
{
    DatenKlasse objekt;
    objekt.setData(7);
    DatenKlasse ziel = tuIrgendwas(objekt);
    DatenKlasse quelle = tuIrgendwas(move(objekt));
    cout << objekt.getData() << endl;
    cout << ziel.getData() << endl;
    ziel = quelle;
    ziel = std::move(quelle);
}

